import { PossibleSelfDefinedType, ParameterlessLazyLoad } from '../types';
declare type Options<T> = Readonly<{
    /**
     * @param u the alternative value to return if the data type does not match and have failed to parse, precompute instead of lazily loaded
     * @returns the parsed value, otherwise the alternative value
     */
    orElseGet: <U>(u: U) => U | T;
    /**
     * @param callBackFunction that is lazily loaded and will be called when it's necessary
     * @returns the parsed value, otherwise the alternative value
     */
    orElseLazyGet: <U>(u: ParameterlessLazyLoad<U>) => U | T;
    /**
     * @returns the parsed value, otherwise the undefined
     * */
    orElseGetUndefined: () => undefined | T;
    /**
     * @returns the parsed value, otherwise the null
     * */
    orElseGetNull: () => null | T;
    /**
     * @name the name of the variable to be shown when Error is thrown
     *
     * ```ts
     * Example:
     * const name = 'example'
     * Expect "name": "example" to have type of "int", actual type is "string"
     * ```
     *
     * @returns the parsed value
     * */
    orElseThrowDefault: (name: string) => T;
    /**
     * @message the message to be shown when Error is thrown
     * @returns the parsed value
     * */
    orElseThrowCustom: (message: string) => T;
}>;
declare type OptionsParameter<T> = Readonly<{
    t: ParameterlessLazyLoad<T>;
    expectedType: PossibleSelfDefinedType;
    isMatchingType: ParameterlessLazyLoad<boolean>;
}>;
declare const createOptions: <T>({ t, expectedType, isMatchingType, }: Readonly<{
    t: ParameterlessLazyLoad<T>;
    expectedType: PossibleSelfDefinedType;
    isMatchingType: ParameterlessLazyLoad<boolean>;
}>) => Readonly<{
    /**
     * @param u the alternative value to return if the data type does not match and have failed to parse, precompute instead of lazily loaded
     * @returns the parsed value, otherwise the alternative value
     */
    orElseGet: <U>(u: U) => T | U;
    /**
     * @param callBackFunction that is lazily loaded and will be called when it's necessary
     * @returns the parsed value, otherwise the alternative value
     */
    orElseLazyGet: <U_1>(u: ParameterlessLazyLoad<U_1>) => T | U_1;
    /**
     * @returns the parsed value, otherwise the undefined
     * */
    orElseGetUndefined: () => T | undefined;
    /**
     * @returns the parsed value, otherwise the null
     * */
    orElseGetNull: () => T | null;
    /**
     * @name the name of the variable to be shown when Error is thrown
     *
     * ```ts
     * Example:
     * const name = 'example'
     * Expect "name": "example" to have type of "int", actual type is "string"
     * ```
     *
     * @returns the parsed value
     * */
    orElseThrowDefault: (name: string) => T;
    /**
     * @message the message to be shown when Error is thrown
     * @returns the parsed value
     * */
    orElseThrowCustom: (message: string) => T;
}>;
declare const createExact: <T, E extends T>({ t, e, expectedType, }: Readonly<{
    t: unknown;
    e: E;
    expectedType: PossibleSelfDefinedType;
}>) => Readonly<{
    /**
     * @param u the alternative value to return if the data type does not match and have failed to parse, precompute instead of lazily loaded
     * @returns the parsed value, otherwise the alternative value
     */
    orElseGet: <U>(u: U) => E | U;
    /**
     * @param callBackFunction that is lazily loaded and will be called when it's necessary
     * @returns the parsed value, otherwise the alternative value
     */
    orElseLazyGet: <U_1>(u: ParameterlessLazyLoad<U_1>) => E | U_1;
    /**
     * @returns the parsed value, otherwise the undefined
     * */
    orElseGetUndefined: () => E | undefined;
    /**
     * @returns the parsed value, otherwise the null
     * */
    orElseGetNull: () => E | null;
    /**
     * @name the name of the variable to be shown when Error is thrown
     *
     * ```ts
     * Example:
     * const name = 'example'
     * Expect "name": "example" to have type of "int", actual type is "string"
     * ```
     *
     * @returns the parsed value
     * */
    orElseThrowDefault: (name: string) => E;
    /**
     * @message the message to be shown when Error is thrown
     * @returns the parsed value
     * */
    orElseThrowCustom: (message: string) => E;
}>;
export { Options, OptionsParameter };
export { createOptions, createExact };
