import { createExact, createOptions } from '../option/index.js';
import { createPrimitiveParser } from './parser.js';
import { getPossibleSelfDefinedType, isEqual } from './util.js';
const parseAsNull = (t) => createPrimitiveParser({
    t,
    expectedType: 'null',
    isMatchingType: () => t === null,
});
const parseAsString = (t) => {
    const isMatchingType = () => isEqual('string', typeof t);
    const orElseGetEmptyString = () => (!isMatchingType() ? '' : t);
    return {
        ...createPrimitiveParser({
            t,
            isMatchingType,
            expectedType: 'string',
        }),
        orElseGetEmptyString,
        exactlyAs: (s) => ({
            ...createExact({
                t,
                e: s,
                expectedType: 'string',
            }),
            orElseGetEmptyString,
        }),
    };
};
const parseAsBoolean = (t) => {
    const expectedType = 'boolean';
    const isMatchingType = () => isEqual(expectedType, typeof t);
    const orElseGet = (b) => (!isMatchingType() ? b : t);
    const orElseGetTrue = () => orElseGet(true);
    const orElseGetFalse = () => orElseGet(false);
    return {
        ...createPrimitiveParser({
            t,
            isMatchingType,
            expectedType: 'boolean',
        }),
        orElseGetTrue,
        orElseGetFalse,
        exactlyAs: (b) => ({
            ...createExact({
                t,
                e: b,
                expectedType: 'boolean',
            }),
            orElseGetTrue,
            orElseGetFalse,
        }),
    };
};
const parseAsNumber = (t) => {
    const expectedType = 'number';
    const isMatchingType = () => isEqual(expectedType, typeof t);
    return {
        ...createPrimitiveParser({
            t,
            expectedType,
            isMatchingType,
        }),
        inRangeOf: (min, max) => {
            if (min > max || max < min) {
                throw new Error(`min must be less than max, max must also be greater than min. Min: ${min}, Max: ${max}`);
            }
            const options = createOptions({
                t: () => t,
                expectedType,
                isMatchingType: () => {
                    if (!isMatchingType()) {
                        return false;
                    }
                    const n = t;
                    return n >= min && n <= max;
                },
            });
            return options;
        },
        exactlyAs: (n) => ({
            ...createExact({
                t,
                e: n,
                expectedType,
            }),
        }),
    };
};
/**
 * Although it can be used to parse anything, this is recommended to be used to parse as a specified non-object type
 * @returns the Options object
 */
const parseAsCustomType = (t, isParseable) => createOptions({
    t: () => t,
    isMatchingType: () => isParseable(t),
    expectedType: getPossibleSelfDefinedType(t),
});
export { parseAsBoolean, parseAsString, parseAsNumber, parseAsNull, parseAsCustomType, };
//# sourceMappingURL=primitive.js.map